//
// Created by philipp on 30.11.17.
//

#ifndef PROTOCOL_FOLLOWERVEHICLE_H
#define PROTOCOL_FOLLOWERVEHICLE_H


#include <mutex>
#include "Vehicle.h"
#include "VehicleEndpoint.h"
#include "PlatoonCreateNetworkMessage.h"

namespace protocol
{

class FollowerVehicle : public Vehicle
{
public:
    using OnUpdatePlatoonConfigCallback = std::function<void(PlatoonConfig)>;

    FollowerVehicle(
        networking::Networking & net,
        VehicleId vehicleId)
        : Vehicle(net, vehicleId, Role::FOLLOWER)
    {}

    void setOnUpdatePlatoonConfigCallback(OnUpdatePlatoonConfigCallback callback)
    { onUpdatePlatoonConfigCallback = callback; }

protected:
    void doOnCreatePlatoon() override;

    void doCreatePlatoon() override;

    void doOnRunningPlatoon() override;

    void doRunningPlatoon() override;

    void doOnLeavingPlatoon() override;

    void doLeavingPlatoon() override;

private:
    VehicleEndpoint leader;
    std::mutex leaderMutex;
    bool leaderFound{false};
    std::vector<VehicleEndpoint>::iterator vehicleEndpointIter;
    OnUpdatePlatoonConfigCallback onUpdatePlatoonConfigCallback{[](PlatoonConfig){}};

    struct Follower
    {
        
    };

    void makePlatoonCreateRequest(VehicleEndpoint vehicleEndpoint);

    PlatoonCreateNetworkMessage makePlatoonCreateResponse(
        const tcp::endpoint & clientEndpoint, const PlatoonCreateNetworkMessage & request);
};

}


#endif //PROTOCOL_FOLLOWERVEHICLE_H
